"use strict";
/**
 *
 * A set of helpers that parses TypeDoc models to be consumed by theme resources.
 *
 * @module
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.getIndexFileName = exports.getSignatureParameters = exports.flattenDeclarations = exports.getMemberTitle = exports.KEYWORD_MAP = exports.getModifier = exports.isGroupKind = exports.isAbsoluteIndex = exports.hasIndex = exports.getProjectDisplayName = exports.getDeclarationType = void 0;
const typedoc_1 = require("typedoc");
const elements_1 = require("../support/elements");
const utils_1 = require("../support/utils");
function getDeclarationType(declaration) {
    if (declaration.signatures) {
        return declaration.signatures[0].type;
    }
    if (declaration.getSignature) {
        return declaration.getSignature.type;
    }
    if (declaration.setSignature) {
        return declaration.setSignature.type;
    }
    return declaration.type;
}
exports.getDeclarationType = getDeclarationType;
function getProjectDisplayName(project, includeVersion) {
    const version = includeVersion && project.packageVersion
        ? ` v${project.packageVersion}`
        : '';
    return `${project.name}${version ? version : ''}`;
}
exports.getProjectDisplayName = getProjectDisplayName;
function hasIndex(reflection) {
    var _a;
    return (_a = reflection.groups) === null || _a === void 0 ? void 0 : _a.some((group) => group.allChildrenHaveOwnDocument());
}
exports.hasIndex = hasIndex;
function isAbsoluteIndex(reflection) {
    var _a;
    return (_a = reflection.groups) === null || _a === void 0 ? void 0 : _a.every((group) => group.allChildrenHaveOwnDocument());
}
exports.isAbsoluteIndex = isAbsoluteIndex;
function isGroupKind(reflection) {
    const groupKinds = [
        typedoc_1.ReflectionKind.Class,
        typedoc_1.ReflectionKind.Interface,
        typedoc_1.ReflectionKind.Enum,
        typedoc_1.ReflectionKind.Function,
        typedoc_1.ReflectionKind.Variable,
        typedoc_1.ReflectionKind.TypeAlias,
    ];
    return groupKinds.includes(reflection.kind);
}
exports.isGroupKind = isGroupKind;
function getModifier(reflection) {
    if (reflection.flags.isAbstract) {
        return 'abstract';
    }
    if (reflection.flags.isPrivate) {
        return 'private';
    }
    if (reflection.flags.isReadonly) {
        return 'readonly';
    }
    if (reflection.flags.isStatic) {
        return 'static';
    }
    if (reflection.flags.isProtected) {
        return 'protected';
    }
    if (reflection.flags.isPublic) {
        return 'public';
    }
    if (reflection.getSignature) {
        return 'get';
    }
    if (reflection.setSignature) {
        return 'set';
    }
    return null;
}
exports.getModifier = getModifier;
exports.KEYWORD_MAP = {
    [typedoc_1.ReflectionKind.Class]: 'class',
    [typedoc_1.ReflectionKind.Interface]: 'interface',
    [typedoc_1.ReflectionKind.Enum]: 'enum',
    [typedoc_1.ReflectionKind.TypeAlias]: 'type',
    [typedoc_1.ReflectionKind.Function]: 'function',
};
function getMemberTitle(reflection) {
    var _a, _b, _c;
    const md = [];
    const name = [];
    if ((reflection === null || reflection === void 0 ? void 0 : reflection.kind) === typedoc_1.ReflectionKind.Class &&
        ((_a = reflection === null || reflection === void 0 ? void 0 : reflection.flags) === null || _a === void 0 ? void 0 : _a.includes('Abstract'))) {
        name.push((0, elements_1.backTicks)('abstract') + ' ');
    }
    name.push(`${reflection.name.startsWith('[') && ((_b = reflection.signatures) === null || _b === void 0 ? void 0 : _b.length)
        ? (0, elements_1.backTicks)(reflection.name)
        : (0, utils_1.escapeChars)(reflection.name)}`);
    if ((_c = reflection.signatures) === null || _c === void 0 ? void 0 : _c.length) {
        name.push('()');
    }
    if (reflection.typeParameters) {
        const typeParameters = reflection.typeParameters
            .map((typeParameter) => typeParameter.name)
            .join(', ');
        name.push(`${`\\<${typeParameters}\\>`}`);
    }
    if (reflection.flags.isOptional) {
        name.push('?');
    }
    if (reflection.isDeprecated && reflection.isDeprecated()) {
        md.push((0, elements_1.strikeThrough)(name.join('')));
    }
    else {
        md.push(name.join(''));
    }
    return md.join(': ');
}
exports.getMemberTitle = getMemberTitle;
function flattenDeclarations(props, includeSignatures = false) {
    const flattenDeclarations = (current) => {
        var _a, _b, _c;
        return (_c = (_b = (_a = current.type) === null || _a === void 0 ? void 0 : _a.declaration) === null || _b === void 0 ? void 0 : _b.children) === null || _c === void 0 ? void 0 : _c.reduce((acc, child) => {
            const childObj = {
                ...child,
                name: `${current.name}.${child.name}`,
            };
            return parseDeclarations(childObj, acc);
        }, []);
    };
    const parseDeclarations = (current, acc) => {
        var _a, _b, _c, _d, _e;
        const shouldFlatten = (_b = (_a = current.type) === null || _a === void 0 ? void 0 : _a.declaration) === null || _b === void 0 ? void 0 : _b.children;
        const isAccessor = current.kind === typedoc_1.ReflectionKind.Accessor;
        if (includeSignatures) {
            if (isAccessor) {
                const accessors = [];
                if (current.getSignature) {
                    accessors.push({
                        ...current,
                        name: `get ${current.name}`,
                        type: current.getSignature.type,
                        comment: (_c = current.getSignature) === null || _c === void 0 ? void 0 : _c.comment,
                    });
                }
                if (current.setSignature) {
                    accessors.push({
                        ...current,
                        name: `set ${current.name}`,
                        type: current.setSignature.type,
                        comment: (_d = current.setSignature) === null || _d === void 0 ? void 0 : _d.comment,
                    });
                }
                return [...acc, ...accessors];
            }
            if ((_e = current.signatures) === null || _e === void 0 ? void 0 : _e.length) {
                const signatures = current.signatures.map((signature) => {
                    return {
                        ...current,
                        name: signature.name,
                        type: signature.type,
                        comment: signature.comment,
                    };
                });
                return [...acc, ...signatures];
            }
        }
        return shouldFlatten
            ? [...acc, current, ...flattenDeclarations(current)]
            : [...acc, current];
    };
    return props.reduce((acc, current) => parseDeclarations(current, acc), []);
}
exports.flattenDeclarations = flattenDeclarations;
function getSignatureParameters(parameters, format = false) {
    const firstOptionalParamIndex = parameters.findIndex((parameter) => parameter.flags.isOptional);
    return ('(' +
        parameters
            .map((param, i) => {
            const paramsmd = [];
            if (param.flags.isRest) {
                paramsmd.push('...');
            }
            const paramItem = `${(0, elements_1.backTicks)(param.name)}${param.flags.isOptional ||
                (firstOptionalParamIndex !== -1 && i > firstOptionalParamIndex)
                ? '?'
                : ''}`;
            paramsmd.push(`${format && parameters.length > 2 ? `\n   ` : ''}${paramItem}`);
            return paramsmd.join('');
        })
            .join(`, `) +
        ')');
}
exports.getSignatureParameters = getSignatureParameters;
function getIndexFileName(reflection, isPackages = false) {
    var _a;
    if (isPackages) {
        return 'packages.md';
    }
    const isModules = (_a = reflection.children) === null || _a === void 0 ? void 0 : _a.every((child) => child.kindOf(typedoc_1.ReflectionKind.Module));
    return isModules ? 'modules.md' : 'globals.md';
}
exports.getIndexFileName = getIndexFileName;
