"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MarkdownTheme = void 0;
const typedoc_1 = require("typedoc");
const maps_1 = require("../options/maps");
const utils_1 = require("../support/utils");
const navigation_context_1 = require("./navigation-context");
const render_context_1 = require("./render-context");
const urls_context_1 = require("./urls-context");
/**
 * This is in-built MarkdownTheme which extends TypeDocs Theme class.
 * This follows the implementation of TypeDoc's [DefaultTheme](https://typedoc.org/api/classes/DefaultThemeRender.html).
 *
 * The {@link render } and {@link getUrls} methods is where the work happens.
 */
class MarkdownTheme extends typedoc_1.Theme {
    /**
     *
     * @param renderer The TypeDoc renderer instance the theme is attached to.
     */
    constructor(renderer) {
        super(renderer);
        this.readmeTemplate = (pageEvent) => {
            return this.getRenderContext(pageEvent).readmeTemplate(pageEvent);
        };
        this.projectTemplate = (pageEvent) => {
            return this.getRenderContext(pageEvent).projectTemplate(pageEvent);
        };
        this.reflectionTemplate = (pageEvent) => {
            return this.getRenderContext(pageEvent).reflectionTemplate(pageEvent);
        };
        this.memberTemplate = (pageEvent) => {
            return this.getRenderContext(pageEvent).memberTemplate(pageEvent);
        };
        // DEPRECATED PROPS WARNING (to delete)
        const deprecatedOptions = [
            'indexPageTitle',
            'memberPageTitle',
            'hideInPageTOC',
        ];
        deprecatedOptions.forEach((option) => {
            if (this.application.options.isSet(option)) {
                this.application.logger.warn(`[typedoc-plugin-markdown] "${option}" is deprecated and will be removed in v4 release. Please see https://www.typedoc-plugin-markdown.org/options#${option.toLowerCase()}`);
            }
        });
        this.textMappings = this.getTextContentMappings();
    }
    getRenderContext(pageEvent) {
        return new render_context_1.MarkdownThemeRenderContext(this, pageEvent, this.application.options);
    }
    getTextContent(key) {
        return this.textMappings[key];
    }
    getTextContentMappings() {
        const textFromOptions = this.application.options.getValue('textContentMappings');
        const knownKeys = Object.keys(maps_1.StaticText);
        for (const key of Object.keys(textFromOptions)) {
            if (!knownKeys.includes(key)) {
                this.application.logger.warn(`[typedoc-plugin-markdown] "${key}" is not a valid "textContentMappings" key. Valid keys are ${knownKeys
                    .map((key) => `"${key}"`)
                    .join(', ')}.`);
            }
        }
        return { ...maps_1.StaticText, ...(textFromOptions || {}) };
    }
    getNavigationContext() {
        return new navigation_context_1.NavigationContext(this, this.application.options.getRawValues(), this.application.renderer);
    }
    getUrlsContext(project) {
        return new urls_context_1.UrlsContext(this, project, this.application.renderer, this.application.options);
    }
    /**
     * Renders a template and page model to a string.
     */
    render(page, template) {
        return template(page);
    }
    getUrls(project) {
        return this.getUrlsContext(project).getUrls();
    }
    getNavigation(project) {
        return this.getNavigationContext().getNavigation(project);
    }
    /**
     * Returns the template mapping for a given reflection kind
     * @param kind
     */
    getTemplateMapping(kind, outputFileStrategyOverride) {
        const outputFileStrategy = outputFileStrategyOverride ||
            this.application.options.getValue('outputFileStrategy');
        const getDirectoryName = (reflectionKind) => {
            const pluralString = typedoc_1.ReflectionKind.pluralString(reflectionKind);
            return (0, utils_1.slugify)(pluralString).toLowerCase();
        };
        // const membersWithOwnFile =
        //   this.application.options.getValue('membersWithOwnFile');
        const mappings = {
            [typedoc_1.ReflectionKind.Module]: {
                template: this.reflectionTemplate,
                directory: null,
                kind: typedoc_1.ReflectionKind.Module,
            },
            [typedoc_1.ReflectionKind.Namespace]: {
                template: this.reflectionTemplate,
                directory: getDirectoryName(typedoc_1.ReflectionKind.Namespace),
                kind: typedoc_1.ReflectionKind.Namespace,
            },
        };
        if (outputFileStrategy === maps_1.OutputFileStrategy.Members //&&
        //membersWithOwnFile?.includes(ReflectionKind[ReflectionKind.Class])
        ) {
            mappings[typedoc_1.ReflectionKind.Class] = {
                template: this.reflectionTemplate,
                directory: getDirectoryName(typedoc_1.ReflectionKind.Class),
                kind: typedoc_1.ReflectionKind.Class,
            };
        }
        if (outputFileStrategy === maps_1.OutputFileStrategy.Members //&&
        //membersWithOwnFile?.includes(ReflectionKind[ReflectionKind.Interface])
        ) {
            mappings[typedoc_1.ReflectionKind.Interface] = {
                isLeaf: false,
                template: this.reflectionTemplate,
                directory: getDirectoryName(typedoc_1.ReflectionKind.Interface),
                kind: typedoc_1.ReflectionKind.Interface,
            };
        }
        if (outputFileStrategy === maps_1.OutputFileStrategy.Members //&&
        //membersWithOwnFile?.includes(ReflectionKind[ReflectionKind.Enum])
        ) {
            mappings[typedoc_1.ReflectionKind.Enum] = {
                template: this.reflectionTemplate,
                directory: getDirectoryName(typedoc_1.ReflectionKind.Enum),
                kind: typedoc_1.ReflectionKind.Enum,
            };
        }
        if (outputFileStrategy === maps_1.OutputFileStrategy.Members //&&
        //membersWithOwnFile?.includes(ReflectionKind[ReflectionKind.Function])
        ) {
            mappings[typedoc_1.ReflectionKind.Function] = {
                template: this.memberTemplate,
                directory: getDirectoryName(typedoc_1.ReflectionKind.Function),
                kind: typedoc_1.ReflectionKind.Function,
            };
        }
        if (outputFileStrategy === maps_1.OutputFileStrategy.Members //&&
        //membersWithOwnFile?.includes(ReflectionKind[ReflectionKind.TypeAlias])
        ) {
            mappings[typedoc_1.ReflectionKind.TypeAlias] = {
                template: this.memberTemplate,
                directory: getDirectoryName(typedoc_1.ReflectionKind.TypeAlias),
                kind: typedoc_1.ReflectionKind.TypeAlias,
            };
        }
        if (outputFileStrategy === maps_1.OutputFileStrategy.Members //&&
        //membersWithOwnFile?.includes(ReflectionKind[ReflectionKind.Variable])
        ) {
            mappings[typedoc_1.ReflectionKind.Variable] = {
                template: this.memberTemplate,
                directory: getDirectoryName(typedoc_1.ReflectionKind.Variable),
                kind: typedoc_1.ReflectionKind.Variable,
            };
        }
        return mappings[kind];
    }
}
exports.MarkdownTheme = MarkdownTheme;
